; hacker port firmware
; version 0.5.1 date 2004-09-06
; part of the hacker port project - http[s]://1stein.schuermans.info/hackerport/
; Copyright (C) 2003-2004 1stein <1stein@schuermans.info>
; Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html

	TITLE		"eeprom procedures for hackerport firmware"

	include		"P18F442.INC"

	include		"IoPins.inc"
	include		"Bits.inc"
	include		"Version.inc"



;===========================================================================================
; EEPROM layout
;===========================================================================================
	#define		EE_ADDR_ID	0x00	; ID in EEPROM (needs 32 bytes)
	#define		EE_ADDR_END	0x20	; unused from here



;===========================================================================================
; imported variables
;===========================================================================================
	extern		BITS0
	extern		A1_REG, B1_REG, C1_REG, D1_REG
	extern		A2_REG, B2_REG, C2_REG, D2_REG
	extern		A4_REG



;===========================================================================================
; imported procedures
;===========================================================================================
	extern		DEBUG_LOG



;===========================================================================================
accessram	UDATA_ACS		; access RAM
;===========================================================================================

; private bit variables
EE_BITS		res	1
#define		EE_WR_ID_ACT	EE_BITS,0	; writing ID from RAM to EEPROM

; next byte of ID to write to EEPROM
EE_WR_ID_ADDR	res	1



;===========================================================================================
ram_ee		UDATA			; normal RAM for EEPROM
;===========================================================================================

; ID to write to EEPROM
EE_WR_ID	res	16



;===========================================================================================
prog		CODE			; normal code
;===========================================================================================

;-------------------------------------------------------------------------------------------
; EEPROM initialization
;-------------------------------------------------------------------------------------------
EE_INIT:
; not writing ID from RAM to EEPROM
	bcf	EE_WR_ID_ACT
; done
	return

;-------------------------------------------------------------------------------------------
; EEPROM task
; - called periodically
; changes:	A1_REG, FSR0, EE*
;-------------------------------------------------------------------------------------------
EE_TASK:
; writing ID from RAM to EEPROM
	btfss	EE_WR_ID_ACT
	bra	EE_TASK_ID_WR_END
; EEPROM still busy
	btfsc	EECON1,WR
	bra	EE_TASK_ID_WR_END
; get next byte from RAM
	lfsr	FSR0,EE_WR_ID		; EEDATA = *(&EE_WR_ID+(EE_WR_ID_ADDR>>1))
	bcf	STATUS,C
	rrcf	EE_WR_ID_ADDR,W
	addwf	FSR0L
	movlw	0
	addwfc	FSR0H
	movff	INDF0,EEDATA
	btfsc	EE_WR_ID_ADDR,0		; complement EEDATA if (EE_WR_ID_ADDR&1)!=0
	comf	EEDATA
; write byte to EEPROM
	movlw	EE_ADDR_ID		; get address of byte in EEPROM
	addwf	EE_WR_ID_ADDR,W
	movwf	EEADR
	movlw	0x04			; configurate EEPROM for writing
	movwf	EECON1
	bcf	INTCON,GIEL		; disable interrupts
	bcf	INTCON,GIEH
	movlw	0x55			; write sequence
	movwf	EECON2
	movlw	0xAA
	movwf	EECON2
	bsf	EECON1,WR
	bsf	INTCON,GIEH		; enable interrupts
	bsf	INTCON,GIEL
; next byte
	incf	EE_WR_ID_ADDR
	movlw	0x20			; if finished, ...
	cpfslt	EE_WR_ID_ADDR
	bcf	EE_WR_ID_ACT		; ..., clear write flag for ID
EE_TASK_ID_WR_END:
; done
	return

;-------------------------------------------------------------------------------------------
; get ID from EEPROM
; input:	FSR0		pointer to 16 byte buffer to write ID to
; output:	-
; changes:	WREG, STATUS, FSR0, FSR1, A1_REG, B1_REG, EE*
;-------------------------------------------------------------------------------------------
EE_GET_ID:
; if currently writing ID
	btfss	EE_WR_ID_ACT
	bra	EE_GET_ID_EE
; return ID from RAM
	lfsr	FSR1,EE_WR_ID		; address of buffer
	movlw	16			; loop 16 times
	movwf	A1_REG
EE_GET_ID_RAM_LOOP:
	movff	POSTINC1,POSTINC0	; copy next byte
	decfsz	A1_REG			; loop
	bra	EE_GET_ID_RAM_LOOP
; done
	return
; read ID from EEPROM and check it
EE_GET_ID_EE:
	movlw	16			; loop 16 times
	movwf	A1_REG
	clrf	EECON1			; configurate EEPROM for reading
	movlw	EE_ADDR_ID		; start address of ID in EEPROM
	movwf	EEADR
	bcf	B1_REG,0		; no error yet
EE_GET_ID_LOOP:
	bsf	EECON1,RD		; read EEPROM
	movff	EEDATA,INDF0
	incf	EEADR			; next address in EEPROM
	bsf	EECON1,RD		; read EEPROM
	comf	EEDATA,W		; if values is not complement of other value, ...
	cpfseq	POSTINC0
	bsf	B1_REG,0		; ..., remember error
	incf	EEADR			; next address in EEPROM
	decfsz	A1_REG			; loop
	bra	EE_GET_ID_LOOP
; done if no error
	btfss	B1_REG,0
	return
; clear ID
	movlw	16			; loop 16 times
	movwf	A1_REG
	movf	POSTDEC0,W		; go to last byte of ID
EE_GET_ID_CLR_LOOP:
	clrf	POSTDEC0		; clear one byte and go one byte back
	decfsz	A1_REG			; loop
	bra	EE_GET_ID_CLR_LOOP
; done
	return

;-------------------------------------------------------------------------------------------
; set ID in EEPROM
; input:	FSR0		pointer to 16 byte buffer with ID
; output:	-
; changes:	WREG, STATUS, FSR0, FSR1, A1_REG, EE*
; schedules the ID to be written into EEPROM
;-------------------------------------------------------------------------------------------
EE_SET_ID:
; save ID to RAM
	lfsr	FSR1,EE_WR_ID		; address of buffer
	movlw	16			; loop 16 times
	movwf	A1_REG
EE_SET_ID_LOOP:
	movff	POSTINC0,POSTINC1	; copy next byte
	decfsz	A1_REG			; loop
	bra	EE_SET_ID_LOOP
; schedule writing ID to EEPROM
	bsf	EE_WR_ID_ACT
	clrf	EE_WR_ID_ADDR
; done
	return



;===========================================================================================
; global variables
;===========================================================================================



;===========================================================================================
; global procedures
;===========================================================================================

	global		EE_INIT, EE_TASK, EE_GET_ID, EE_SET_ID



	end

